''  jm_boxes_std_scroll.bas - jm_gui
''	Copyright (C) 2005-2007 Jeff Marshall (coder@execulink.com)
''
''  This software is provided 'as-is', without any express or implied
''  warranty.  In no event will the authors be held liable for any damages
''  arising from the use of this software.
''
''  Permission is granted to anyone to use this software for any purpose,
''  including commercial applications, and to alter it and redistribute it
''  freely, subject to the following restrictions:
''
''  1. The origin of this software must not be misrepresented; you must not
''     claim that you wrote the original software. If you use this software
''     in a product, an acknowledgment in the product documentation would be
''     appreciated but is not required.
''  2. Altered source versions must be plainly marked as such, and must not
''     be misrepresented as being the original software.
''  3. This notice may not be removed or altered from any source 
''     distribution.
''

#include "inc1/jm_gui_rt.bi"
#include "jm_gui_base.bi"
#include "jm_boxes.bi"
#include "../jm_font/jm_font.bi"
#include "jm_boxes_std_scroll.bi"

'' -------------------------------------------------------------------
'' BOX CREATION
'' -------------------------------------------------------------------

Function UI_CreateVScroll alias "UI_CreateVScroll" (ByVal ownerbox As UI_BOX_PTR, byval id As integer, _
	byval x As Integer, byval y As Integer, byval w As Integer, byval h As Integer, _
	ByVal BoxProc As BOXPROC_FUNC_T, ByVal flags As Integer ) As UI_BOX_PTR
	return UI_CreateBox(ownerbox, UI_BOX_VSCROLL, id, x, y, w, h, BoxProc, @UI_DefVScrollProc_cb, sizeof(UI_SCROLL_T), flags)
End Function

'' -------------------------------------------------------------------
'' DRAWING
'' -------------------------------------------------------------------

Private Sub UI_DrawStdVScroll alias "UI_DrawStdVScroll" (ByVal r As UI_RECT_T Ptr, ByVal flags As Integer, ByVal d As UI_SCROLL_T Ptr)
	Dim As Integer xx,yy,ww,hh
	Dim As UI_RECT_T rtop, rmid, rbot, rscr

	UI_SetRect @rtop, r->x, r->y, r->w, r->w
	UI_SetRect @rbot, r->x, r->y + r->h - r->w, r->w, r->w
	UI_SetRect @rmid, r->x, r->y + r->w, r->w, r->h - r->w - r->w

	If d->flags = 1 Then
		' FIXME: Use image instead of character
		UI_DrawStdButton @rtop, (flags And UI_BOX_FLAG_DOWN), chr(30)
	Else
		' FIXME: Use image instead of character
		UI_DrawStdButton @rtop, 0, chr(30)
	End If

	UI_DrawFrame @rmid, UI_FRAMESTYLE_THIN or UI_FRAMESTYLE_FILL

	If d->flags = 2 Then
		UI_DrawStdButton @rbot, (flags And UI_BOX_FLAG_DOWN), chr(31)
	Else
		UI_DrawStdButton @rbot, 0, chr(31)
	End If

	If d->imax - d->imin > 0 Then
		hh = rmid.h - (r->w - 2)
		if hh > (r->w - 2) Then
			yy = (csng(hh) / csng((d->imax - d->imin))) * csng(d->ival)
			UI_SetRect @rscr, r->x + 1, rmid.y + yy, r->w-2, r->w-2
			UI_DrawStdButton @rscr, 0, ""
		end if
	end if

End Sub

'' -------------------------------------------------------------------
'' DEFAULT BOXPROCS
'' -------------------------------------------------------------------

Function UI_DefVScrollProc_cb alias "UI_DefVScrollProc_cb"  (ByVal box As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer
	Dim r As UI_RECT_T
	Dim d As UI_SCROLL_T Ptr = UI_GetBoxExtraBytesPtr(box)
	Dim b as Integer = 0

	If (UI_CallBoxProc(box, msgid, msgdata)) Then
		Select Case msgid
		Case UI_MSG_PAINT
			UI_GetScreenCoords box, @r
			UI_DrawStdVScroll @r, box->flags, d
		Case UI_MSG_MOUSE_DOWN
			If (box->flags And UI_BOX_FLAG_DISABLED) = 0 Then
				If UI_GetCapture() = NULL Then
  				UI_GetBoxCoords box, @r
					If (msgdata->my < r.w) Then
						d->flags = 1
					ElseIf (msgdata->my >= r.h - r.w) Then
						d->flags = 2
					Else
						d->flags = 3
					EndIf
					UI_SetCapture box
					box->flags or= UI_BOX_FLAG_DOWN
				End If
			End If
		Case UI_MSG_MOUSE_UP
			If (box->flags And UI_BOX_FLAG_DISABLED) = 0 Then
				If UI_GetCapture() = box Then
  				UI_GetBoxCoords box, @r
					If (msgdata->my < r.w) Then
						If d->flags = 1 Then
						  if d->ival > d->imin Then
								d->ival -= 1
								b = 1
							End If
						end if
					ElseIf (msgdata->my >= r.h - r.w) Then
						if d->flags = 2 then
						  if d->ival < d->imax Then
								d->ival += 1
								b = 1
							End If
						end if
					Else
						if d->flags = 3 then
						else
						end if
					EndIf
					UI_SetCapture box
					box->flags or= UI_BOX_FLAG_DOWN
					d->flags = 0
					if (b) Then
						UI_CallBoxProc(box, UI_MSG_CHANGE, msgdata)
					End If
				End If
			End If
			box->flags and= (not UI_BOX_FLAG_DOWN)
			UI_ReleaseCapture
		Case UI_MSG_MOUSE_MOVE
			If UI_GetCapture() = box Then
				UI_GetBoxCoords box, @r
				If UI_IsPointInRect(@r, msgdata->mx, msgdata->my) Then
					If d->flags = 1 Then
						If (msgdata->my < r.w) Then
							box->flags or= UI_BOX_FLAG_DOWN
						Else
							box->flags and= (not UI_BOX_FLAG_DOWN)
						End If
					Elseif d->flags = 2 Then
						If (msgdata->my >= r.h - r.w) Then
							box->flags or= UI_BOX_FLAG_DOWN
						Else
							box->flags and= (not UI_BOX_FLAG_DOWN)
						End If
					Else
						Dim yy As integer, iold As Integer
						box->flags or= UI_BOX_FLAG_DOWN
						yy = (d->imax - d->imin) / (r.h - r.w - r.w) * (msgdata->my - r.w)
						if yy < d->imin then
							yy = d->imin
						elseif yy > d->imax then
							yy = d->imax
						end if
						If yy <> d->ival Then
							d->ival = yy
							UI_CallBoxProc(box, UI_MSG_CHANGE, msgdata)						
						End If
					End If
				Else
					box->flags and= (not UI_BOX_FLAG_DOWN)
				End If
			End If
		Case Else
			return 1
		End Select
	else
	  return 1 '' Msg not processed
	End If

End Function
