''  jm_list_buffer.bas - jm_gui
''	Copyright (C) 2005-2007 Jeff Marshall (coder@execulink.com)
''
''  This software is provided 'as-is', without any express or implied
''  warranty.  In no event will the authors be held liable for any damages
''  arising from the use of this software.
''
''  Permission is granted to anyone to use this software for any purpose,
''  including commercial applications, and to alter it and redistribute it
''  freely, subject to the following restrictions:
''
''  1. The origin of this software must not be misrepresented; you must not
''     claim that you wrote the original software. If you use this software
''     in a product, an acknowledgment in the product documentation would be
''     appreciated but is not required.
''  2. Altered source versions must be plainly marked as such, and must not
''     be misrepresented as being the original software.
''  3. This notice may not be removed or altered from any source 
''     distribution.
''

#include "jm_list_buffer.bi"
#include "../jm_string/jm_string.bi"

private function list_buffer_resize alias "list_buffer_resize" (byval lst as LIST_BUFFER_T ptr, byval size as integer) as integer
	dim newlst as any ptr
	function = 0
	if (size > lst->msize) then
		newlst = reallocate(lst->mitems, sizeof(LIST_BUFFER_ITEM_T) * lst->msize)
		if (newlst) then
			lst->msize = size
			lst->mitems = newlst
			function = 1
		end if
	end if
	'' FIXME: allow resize smaller
end function

function LIST_BUFFER_Create alias "LIST_BUFFER_Create" (byval startsize as integer) as LIST_BUFFER_T ptr
	dim lst as LIST_BUFFER_T ptr = 0
	if startsize > 0 then
		lst = allocate(sizeof(LIST_BUFFER_T))
		if (lst) then
			lst->msize = startsize
			lst->mcount = 0
			lst->mitems = allocate(sizeof(LIST_BUFFER_ITEM_T) * lst->msize)
		end if
	end if
	function = lst
end function

sub LIST_BUFFER_Destroy alias "LIST_BUFFER_Destroy" (byval lst as LIST_BUFFER_T ptr)
	dim i as integer
	if (lst) then
		if (lst->mitems) then
			for i = 0 to lst->mcount - 1
				if lst->mitems[i].mtext then
					deallocate lst->mitems[i].mtext
				end if
			next i
			deallocate lst->mitems
		end if
		deallocate lst
	end if
end sub

sub LIST_BUFFER_Clear alias "LIST_BUFFER_Clear" (byval lst as LIST_BUFFER_T ptr)
	dim i as integer
	if (lst) then
		if (lst->mitems) then
			for i = 0 to lst->mcount - 1
				if lst->mitems[i].mtext then
					deallocate lst->mitems[i].mtext
				end if
			next i
		end if
		lst->mindex = -1
		lst->mcount = 0
		' FIXME: condition to trigger a shrink in memory size?
	end if
end sub

function LIST_BUFFER_GetListCount alias "LIST_BUFFER_GetListCount" (byval lst as LIST_BUFFER_T ptr) as integer
	function = 0
	if (lst) then
		function = lst->mcount
	end if
end function

function LIST_BUFFER_GetListIndex alias "LIST_BUFFER_GetListIndex" (byval lst as LIST_BUFFER_T ptr) as integer
	function = -1
	if (lst) then
		function = lst->mindex
	end if
end function

sub LIST_BUFFER_SetListIndex alias "LIST_BUFFER_SetListIndex" (byval lst as LIST_BUFFER_T ptr, byval index as integer)
	if (lst) then
		if index >= -1 and index < lst->mcount then
			lst->mindex = index
		end if
	end if
end sub

function LIST_BUFFER_GetText alias "LIST_BUFFER_GetText" (byval lst as LIST_BUFFER_T ptr, byval index as integer) as zstring ptr
	function = 0
	if (lst) then
		if (index >= 0) then
			if (index < lst->mcount) then
				function = lst->mitems[index].mtext
			end if
		end if
	end if
end function

sub LIST_BUFFER_SetText alias "LIST_BUFFER_SetText" (byval lst as LIST_BUFFER_T ptr, byval index as integer, byval txt as zstring ptr)
	if (lst) then
		if (index >= 0) then
			if (index < lst->mcount) then
				if (txt) then
					dim n as integer = zstr_Len(txt)
					lst->mitems[index].mtext = reallocate(lst->mitems[index].mtext, n + 1)
					zstr_CopyN(lst->mitems[index].mtext, txt, n)
				else
					lst->mitems[index].mtext = NULL
				end if
			end if
		end if
	end if
end sub

function LIST_BUFFER_GetData alias "LIST_BUFFER_GetValue" (byval lst as LIST_BUFFER_T ptr, byval index as integer) as integer
	function = 0
	if (lst) then
		if (index >= 0) then
			if (index < lst->mcount) then
				function = lst->mitems[index].mdata
			end if
		end if
	end if
end function

sub LIST_BUFFER_SetData alias "LIST_BUFFER_SetValue" (byval lst as LIST_BUFFER_T ptr, byval index as integer, byval idata as integer)
	if (lst) then
		if (index >= 0) then
			if (index < lst->mcount) then
				lst->mitems[index].mdata = idata
			end if
		end if
	end if
end sub

sub LIST_BUFFER_AddItem alias "LIST_BUFFER_AddItem" (byval lst as LIST_BUFFER_T ptr, byval index as integer, byval txt as zstring Ptr, byval idata as integer)
	if (lst) then
		if lst->mcount >= lst->msize then
			if lst->msize < 8 then
				list_buffer_resize lst, 8
			else
				list_buffer_resize lst, lst->msize * 2
			end if
		end if
		if lst->mcount < lst->msize then
			if index < 0 then 
				index = lst->mcount
			elseif index > lst->mcount then
				index = lst->mcount
			end if
			if lst->mcount - index > 0 then
				mstr_Move lst->mitems + index + 1,lst->mitems + index, (lst->mcount - index) * sizeof(LIST_BUFFER_ITEM_T)
			end if
			lst->mitems[index].mtext = 0
			lst->mitems[index].mdata = idata
			LIST_BUFFER_SetText lst, index, txt
			lst->mcount += 1
		end if
	end if
end sub

sub LIST_BUFFER_RemoveItem alias "LIST_BUFFER_RemoveItem" (byval lst as LIST_BUFFER_T ptr, byval index as integer)
	if (lst) then
		if (index >= 0) and (index < lst->mcount) then
			if lst->mitems[index].mtext then
				deallocate lst->mitems[index].mtext
			end if
			if index < lst->mcount - 1 then
				mstr_Move lst->mitems + index, lst->mitems + index + 1, (lst->mcount - index - 1) * sizeof(LIST_BUFFER_ITEM_T)
			end if
			lst->mcount -= 1
			if index >= lst->mcount then
				lst->mindex = lst->mcount - 1
			elseif index < lst->mindex then
				lst->mindex -= 1
			end if
		end if
	end if
end sub
