''  jm_boxes_mygl.bas - jm_gui
''	Copyright (C) 2005-2007 Jeff Marshall (coder@execulink.com)
''
''  This software is provided 'as-is', without any express or implied
''  warranty.  In no event will the authors be held liable for any damages
''  arising from the use of this software.
''
''  Permission is granted to anyone to use this software for any purpose,
''  including commercial applications, and to alter it and redistribute it
''  freely, subject to the following restrictions:
''
''  1. The origin of this software must not be misrepresented; you must not
''     claim that you wrote the original software. If you use this software
''     in a product, an acknowledgment in the product documentation would be
''     appreciated but is not required.
''  2. Altered source versions must be plainly marked as such, and must not
''     be misrepresented as being the original software.
''  3. This notice may not be removed or altered from any source 
''     distribution.
''

#include "inc1/jm_gui_rt.bi"
#include "jm_gui_base.bi"
#include "jm_boxes.bi"
#include "mygl.bi"

declare function UI_MakeRoundBorderImage alias "UI_MakeRoundBorderImage" (byval clr as integer,byval invert as integer, byval sz as integer) As any ptr
declare function UI_MakeSquareBorderImage alias "UI_MakeSquareBorderImage" (byval clr as integer,byval invert as integer, byval sz as integer) As any ptr

const bs as integer = 16.0							' border size
const sz as integer = bs * 3.0          ' texture size
const th as integer = 4                	' displayed thickness

const mBlend = &Hff000000

Dim shared As myglTexture brdrup, brdrdn, squareup, squaredn

'' Stretch blit ranged using mygl_texture_blit_scaled for actual blit
private sub StretchBlitRanged alias "StretchBlitRanged" (byval x1 as integer,byval y1 as integer,byval x2 as integer,byval y2 as integer, _
  byval src_min_x as integer,byval src_min_y as integer,byval src_max_x as integer,byval src_max_y as integer, _
	byval surf As myglTexture, byval blit_mode as MYGL_BLIT_TYPE, byval blending as integer, byval tinting as myglColor)

	dim scalex as single = csng(x2 - x1 + 1.0) / csng(abs(src_max_x - src_min_x) + 1)
	dim scaley as single = csng(y2 - y1 + 1.0) / csng(abs(src_max_y - src_min_y) + 1)

	dim c as integer = tinting or blending
	cast(byte ptr, @c)[0] = ((tinting shr 16) and 255)
	cast(byte ptr, @c)[2] = (tinting and 255)

	MyGL_Texture_BlitScaled( surf, x1, y1, src_min_x,src_min_y,src_max_x,src_max_y, scalex, scaley, blit_mode, c )
end sub

private function MakeRoundBorderTex alias "MakeRoundBorderTex" (byval clr as integer,byval invert as integer) As myglTexture
	dim a as ubyte ptr, ret as myglTexture = 0
	a = UI_MakeRoundBorderImage(clr, invert, sz)
	if (a) then
		MyGL_Texture_LoadMem Cast(ubyte ptr, a) + 4, ret, sz, sz, MYGL_PIXEL_RGBA, 1, 0, MYGL_TEXTURE_LINEAR
		Deallocate a
	end if
	function = ret
end function		

private function MakeSquareBorderTex alias "MakeSquareBorderTex" (byval clr as integer,byval invert as integer) As myglTexture
	dim a as ubyte ptr, ret as myglTexture = 0
	a = UI_MakeSquareBorderImage(clr, invert, sz)
	if (a) then
		MyGL_Texture_LoadMem Cast(ubyte ptr, a) + 4, ret, sz, sz, MYGL_PIXEL_RGBA, 1, 0, MYGL_TEXTURE_LINEAR
		Deallocate a
	end if
	function = ret
end function		

'' draw border @ (x,y)-step(w,h) using surface `s' with a thickness of
private sub DrawBorder alias "DrawBorder" (byval x as integer, byval y as integer, byval w as integer, byval h as integer, _
	byval s as myglTexture, ByVal thickness as integer, byval blending as integer = &hff000000, _
	byval fill as integer = 1, byval outside as integer = 0, byval Tinting as integer = &hffffff)

	dim as integer s1 = 0
	dim as integer s2 = bs - 1 
	dim as integer s3 = bs
	dim as integer s4 = sz - 1 - bs - 1
	dim as integer s5 = sz - 1 - bs
	dim as integer s6 = sz - 1

	dim as integer x1 = x 
	dim as integer x2 = x - 1 + thickness
	dim as integer x3 = x + thickness
	dim as integer x4 = x + w - 1 - thickness
	dim as integer x5 = x + w - thickness
	dim as integer x6 = x + w - 1
	
	dim as integer y1 = y 
	dim as integer y2 = y - 1 + thickness
	dim as integer y3 = y + thickness
	dim as integer y4 = y + h - 1 - thickness
	dim as integer y5 = y + h - thickness
	dim as integer y6 = y + h - 1

	if (outside) then
		x1 -= thickness : x2 -= thickness : x3 -= thickness
		x4 += thickness : x5 += thickness : x6 += thickness
		y1 -= thickness : y2 -= thickness : y3 -= thickness
		y4 += thickness : y5 += thickness : y6 += thickness
	end if

	'' Top
	StretchBlitRanged  x1,y1,x2,y2,s1,s1,s2,s2, s, MYGL_BLIT_BLEND, blending, tinting
	StretchBlitRanged  x3,y1,x4,y2,s3,s1,s4,s2, s, MYGL_BLIT_BLEND, blending, tinting
	StretchBlitRanged  x5,y1,x6,y2,s5,s1,s6,s2, s, MYGL_BLIT_BLEND, blending, tinting

	'' Mid
	StretchBlitRanged  x1,y3,x2,y4,s1,s3,s2,s4, s, MYGL_BLIT_BLEND, blending, tinting
	if (fill) then
	StretchBlitRanged  x3,y3,x4,y4,s3,s3,s4,s4, s, MYGL_BLIT_BLEND, blending, tinting
	end if
	StretchBlitRanged  x5,y3,x6,y4,s5,s3,s6,s4, s, MYGL_BLIT_BLEND, blending, tinting

	'' Bottom
	StretchBlitRanged  x1,y5,x2,y6,s1,s5,s2,s6, s, MYGL_BLIT_BLEND, blending, tinting
	StretchBlitRanged  x3,y5,x4,y6,s3,s5,s4,s6, s, MYGL_BLIT_BLEND, blending, tinting
	StretchBlitRanged  x5,y5,x6,y6,s5,s5,s6,s6, s, MYGL_BLIT_BLEND, blending, tinting

end sub

Sub UI_DrawInit alias "UI_DrawInit" ()
	squareup = MakeSquareBorderTex(Rgba(255,255,255,255),0)
	squaredn = MakeSquareBorderTex(Rgba(255,255,255,255),1)
	brdrup = MakeRoundBorderTex(Rgba(255,255,255,255),0)
	brdrdn = MakeRoundBorderTex(Rgba(255,255,255,255),1)
End Sub

'' -------------------------------------------------------------------
'' DRAWING PRIMITIVES
'' -------------------------------------------------------------------

''private Sub UI_DrawLine(ByVal x1 As Integer, ByVal y1 As Integer, ByVal x2 As Integer, ByVal y2 As Integer, ByVal clr As Integer)
''	_line x1,y1,x2,y2,clr
''End Sub
''private Sub UI_DrawHLine(ByVal x As Integer, ByVal y As Integer, Byval w As Integer, ByVal clr As Integer)
''	_line x,y,x+w-1,y,clr
''End Sub
''private Sub UI_DrawVLine(ByVal x As Integer, ByVal y As Integer, Byval h As Integer, ByVal clr As Integer)
''	_line x,y,x,y+h-1,clr
''End Sub
''private Sub UI_DrawRect(ByVal x As Integer, ByVal y As Integer, Byval w As Integer, ByVal h As Integer, ByVal clr As Integer)
''	_box x,y,x+w-1,y+h-1,clr
''End Sub
private Sub UI_DrawFillRect alias "UI_DrawFillRect" (ByVal x As Integer, ByVal y As Integer, Byval w As Integer, ByVal h As Integer, ByVal clr As Integer)
	MyGL_Primitive_RectFilled( x,y,x+w-1,y+h-1,clr )
End Sub

Sub UI_DrawCheckBox alias "UI_DrawCheckBox" (ByVal r As UI_RECT_T Ptr, ByVal Checked As Integer, ByVal clr As Integer)
	Dim r1 As UI_RECT_T
	UI_SetRect @r1, r->x, r->y + (r->h - r->w) \ 2, r->w, r->w
	If (Checked) Then
		UI_DrawFrame @r1, UI_FRAMESTYLE_THIN or UI_FRAMESTYLE_FILL or UI_FS_BTN_DN
	else
		UI_DrawFrame @r1, UI_FRAMESTYLE_THIN or UI_FRAMESTYLE_FILL or UI_FS_BTN_UP
	End If
End Sub

Sub UI_DrawFrame alias "UI_DrawFrame"  (ByVal r As UI_RECT_T Ptr, ByVal iStyle as integer)
	Dim As Integer c1=0,c2=0
	Dim As Integer x = r->x
	Dim As Integer y = r->y
	Dim As Integer w = r->w
	Dim As Integer h = r->h

	select case (iStyle And 7)
	case UI_FRAMESTYLE_UP
		DrawBorder x,y,w,h,brdrup,th,mBlend,(iStyle And UI_FRAMESTYLE_FILL), (iStyle And UI_FRAMESTYLE_OUTSIDE),UI_GetSysColor(UI_SYSCLR_FRAME_FACE)

	case UI_FRAMESTYLE_DOWN
		DrawBorder x,y,w,h,brdrdn,th,mBlend,(iStyle And UI_FRAMESTYLE_FILL), (iStyle And UI_FRAMESTYLE_OUTSIDE),UI_GetSysColor(UI_SYSCLR_FRAME_FACE)

	case UI_FRAMESTYLE_THIN, (UI_FRAMESTYLE_THIN or UI_FRAMESTYLE_UP)
		DrawBorder x,y,w,h,squareup,th,mBlend,(iStyle And UI_FRAMESTYLE_FILL), (iStyle And UI_FRAMESTYLE_OUTSIDE),UI_GetSysColor(UI_SYSCLR_FRAME_FACE)

	case (UI_FRAMESTYLE_THIN or UI_FRAMESTYLE_DOWN)
		DrawBorder x,y,w,h,squaredn,th,mBlend,(iStyle And UI_FRAMESTYLE_FILL), (iStyle And UI_FRAMESTYLE_OUTSIDE),UI_GetSysColor(UI_SYSCLR_FRAME_FACE)

	case UI_FRAMESTYLE_EDGE
		DrawBorder x,y,w,h,brdrdn,th,mBlend,(iStyle And UI_FRAMESTYLE_FILL), (iStyle And UI_FRAMESTYLE_OUTSIDE),UI_GetSysColor(UI_SYSCLR_FRAME_FACE)

	case UI_FRAMESTYLE_THICK
		DrawBorder x,y,w,h,brdrup,th,mBlend,(iStyle And UI_FRAMESTYLE_FILL), (iStyle And UI_FRAMESTYLE_OUTSIDE),UI_GetSysColor(UI_SYSCLR_FRAME_FACE)

	end Select

End Sub

Sub UI_DrawCaret alias "UI_DrawCaret" (byval x as integer, byval y as integer, byval w as integer, byval h as integer, byval clr as integer)
	UI_DrawFillRect x, y, w, h, clr
End Sub
