''  jm_image_load_bmp.bas - jm_gui
''	Copyright (C) 2005-2007 Jeff Marshall (coder@execulink.com)
''
''  This software is provided 'as-is', without any express or implied
''  warranty.  In no event will the authors be held liable for any damages
''  arising from the use of this software.
''
''  Permission is granted to anyone to use this software for any purpose,
''  including commercial applications, and to alter it and redistribute it
''  freely, subject to the following restrictions:
''
''  1. The origin of this software must not be misrepresented; you must not
''     claim that you wrote the original software. If you use this software
''     in a product, an acknowledgment in the product documentation would be
''     appreciated but is not required.
''  2. Altered source versions must be plainly marked as such, and must not
''     be misrepresented as being the original software.
''  3. This notice may not be removed or altered from any source 
''     distribution.
''

#include "crt/stdio.bi"
#include "jm_image.bi"

#define BI_RGB		0
#define BI_BITFIELDS	3

Type BITMAPFILEHEADER field=1
        bfType As Short
        bfSize As Integer
        bfReserved1 As Short
        bfReserved2 As Short
        bfOffBits As Integer
End Type

Type BITMAPINFOHEADER field=1
        biSize As Integer
        biWidth As Integer
        biHeight As Integer
        biPlanes As Short
        biBitCount As Short
        biCompression As Integer
        biSizeImage As Integer
        biXPelsPerMeter As Integer
        biYPelsPerMeter As Integer
        biClrUsed As Integer
        biClrImportant As Integer
End Type

Private function load_bmp_gs8(byval f as FILE Ptr, byval bitdepth as integer) as Any Ptr
	Dim as ubyte ptr img = NULL
	Dim as ubyte ptr dst = NULL
	Dim as ubyte ptr tmp
	Dim as BITMAPFILEHEADER bmfh
	Dim as BITMAPINFOHEADER bmih
	Dim as integer bpp, i, j, k, n, dw, h
	
	Select case bitdepth
	case 8
		bpp = 1
	case 16
		bpp = 2
	case 24,32
		bpp = 4
	case else
		return NULL
	end select
	
	if (fread(@bmfh, sizeof(BITMAPFILEHEADER), 1, f) = 0) then
		return NULL '' file io error
	end if
	
	if (bmfh.bfType <> 19778) then '' 'BM'
		return NULL '' not a bitmap file
	end if

	if (fread(@bmih, sizeof(BITMAPINFOHEADER), 1, f) = 0) then
		return NULL '' unable to read
	end if
	
	if (bmih.biSize <> sizeof(BITMAPINFOHEADER)) then
		return NULL '' unknown header size
	end if

  if ((bmih.biBitCount <> 8) or (bmih.biPlanes <> 1) or (bmih.biCompression <> BI_RGB)) then
		return NULL '' not an 8 bit grayscale.
	end if

	n = (bmih.biWidth + 3) and (not 3) '' Image bytes are DWORD aligned
	tmp = Allocate(n)

	if (tmp = 0) then
		return NULL
	end if

	if (bmih.biHeight < 0) then
		h = -bmih.biHeight
	else
		h = bmih.biHeight
	end if

	img = Allocate(bmih.biWidth * h * bpp + 4)
	if (img) then
		Cast(ushort ptr,img)[0] = (bmih.biWidth shl 3) or bpp
		Cast(ushort ptr,img)[1] = h
		
		if (bmih.biHeight < 0) then
			dw = bmih.biWidth * bpp
			dst = img + 4
		else
			dw = -bmih.biWidth * bpp
			dst = img + 4 +	bmih.biWidth * h * bpp + dw
		end if

		fseek(f, bmfh.bfOffBits, SEEK_SET)
		for i=0 to h - 1
			if (fread(tmp, n, 1, f) = 0) then
				exit for
			end if
			for j = 0 to bmih.biWidth - 1
				for k = 0 to bpp - 1
					dst[j * bpp + k] = tmp[j]
				next k
			next j
			dst += dw
		next i
	end if
	return img
end function

function IMAGE_LoadFile alias "IMAGE_LoadFile" (byval filename as zstring ptr, byval bitdepth as integer) as any ptr
	dim as any ptr img
	dim f as FILE ptr
	f = fopen(filename, "rb")
	if (f) then
		img = load_bmp_gs8(f, bitdepth)
		fclose(f)
	end if
	return img
end function
