#include <string.h>
#include "jm_string.h"

/*
''  Copyright (C) 2005-2007, Jeff Marshall (coder@execulink.com)
''
''  This software is provided 'as-is', without any express or implied
''  warranty.  In no event will the authors be held liable for any damages
''  arising from the use of this software.
''
''  Permission is granted to anyone to use this software for any purpose,
''  including commercial applications, and to alter it and redistribute it
''  freely, subject to the following restrictions:
''
''  1. The origin of this software must not be misrepresented; you must not
''     claim that you wrote the original software. If you use this software
''     in a product, an acknowledgment in the product documentation would be
''     appreciated but is not required.
''  2. Altered source versions must be plainly marked as such, and must not
''     be misrepresented as being the original software.
''  3. This notice may not be removed or altered from any source 
''     distribution.
*/

#define STRINGCALL

struct dstr_desc_t
{
	int size;			// size of memory block - not including header
	int length;		// size of the contained string
	char data[];  // the string
};
typedef struct dstr_desc_t DSTR_DESC;
#define DSTR_DESC_SIZE sizeof(struct dstr_desc_t)

static DSTR_DESC dstr_Null = { 0, 0, "\0" };

STRINGCALL char * zstr_Alloc(int size)
{
	char * r = NULL;
	if (size > 0)
	{
		if (r = malloc(size))
			*r = '\0';
	}
	return r;
}

STRINGCALL char * zstr_Free(char * s)
{
	if (s)
		free(s);
}

STRINGCALL int zstr_Len(char * s)
{
	int n = 0;
	if (s)
	{
		while (*s++)
			n++;
	}
	return n;
}

STRINGCALL char * zstr_Copy(char * dst, char * src)
{
	if (dst)
	{
		char * p = dst;
		if (src)
			while (*p++ = *src++);
		*p = '\0';
	}
	return(dst);
}

STRINGCALL char * zstr_CopyN(char * dst, char * src, int size)
{
	if (size < 0)
		size = 0;

	if (dst)
	{
		char * p = dst;
		if (src)
			while (*p++ = *src++);
		p[size] = '\0';
	}
	return(dst);
}

/*
STRINGCALL char * zstr_AllocFill(int size, int ch)
{
	char * r = NULL;
	if (size > 0)
	{
		if (r = zstr_Alloc(size))
		{
			char * p = r;
			while(size--)
				*p++ = ch & 255;
			*p = '\0';
		}
	}
	return r;
}
*/

STRINGCALL void dstr_Free(DSTR p)
{
	if (p)
		free(p - DSTR_DESC_SIZE);
}

static DSTR_DESC * dstr_alloc(int maxsize)
{
	if (maxsize > 0)
	{
		maxsize = ((maxsize + 1 + 3)&~3);
		DSTR_DESC * d = malloc( maxsize + DSTR_DESC_SIZE );
		if (d)
		{
			d->size = maxsize - 1;
			d->length = 0;
			d->data[d->length] = '\0';
			return(d);
		}
	}
	return(NULL);
}

STRINGCALL DSTR dstr_AllocNew(int maxsize)
{
	DSTR_DESC * d = dstr_alloc(maxsize);	
	if (d)
		return((DSTR)(d + 1));
	return((DSTR)(&dstr_Null + 1));
}

STRINGCALL DSTR dstr_AllocCopyZ(char * src)
{
	int n = zstr_Len(src);
	if (n)
	{
		DSTR_DESC * d = dstr_alloc(n);
		if (d)
		{
			char * dst = d->data;
			while (*dst++ = *src++);
			d->length = n;
			return((DSTR)(d + 1));
		}
	}
	return((DSTR)(&dstr_Null + 1));
}

STRINGCALL DSTR dstr_AssignZ(DSTR dst, char * src)
{
	if (dst)	
	{
		DSTR_DESC * d = (DSTR_DESC *)(dst - DSTR_DESC_SIZE);
		int n = zstr_Len(src);
		if (n <= d->size)
		{
			zstr_Copy(d->data, src);
			d->length = n;
			return(dst);
		}
		dstr_Free(dst);
	}
	return dstr_AllocCopyZ(src);
}

STRINGCALL void mstr_Copy(void * dst, void * src, int bytes)
{
	memcpy(dst, src, bytes);	
}

STRINGCALL void mstr_Move(void * dst, void * src, int bytes)
{
	memmove(dst, src, bytes);
}

STRINGCALL void mstr_Clear(void * dst, int bytes)
{
	memset(dst, 0, bytes);
}

STRINGCALL void mstr_Fill(void * dst, int ch, int bytes)
{
	memset(dst, ch, bytes);
}
